@@ -1,10 +1,11 @@
 from dataclasses import field
 from typing import List, Optional, Set
 
-from pydantic import root_validator
+from pydantic import model_validator
 from pydantic.dataclasses import dataclass
 
 from metaphor.common.base_config import BaseConfig
+from metaphor.common.dataclass import ConnectorConfig
 from metaphor.common.filter import DatasetFilter
 from metaphor.common.tag_matcher import TagMatcher
 from metaphor.common.utils import must_set_exactly_one
@@ -14,7 +15,7 @@ from metaphor.common.utils import must_set_exactly_one
 DEFAULT_QUERY_LOG_FETCH_SIZE = 1000
 
 
-@dataclass
+@dataclass(config=ConnectorConfig)
 class BigQueryCredentials:
     """Credentials used to connect to BigQuery"""
 
@@ -38,7 +39,7 @@ class BigQueryCredentials:
     token_uri: str = "https://oauth2.googleapis.com/token"
 
 
-@dataclass
+@dataclass(config=ConnectorConfig)
 class BigQueryQueryLogConfig:
     # Number of days back of query logs to fetch, if 0, don't fetch query logs
     lookback_days: int = 1
@@ -56,10 +57,10 @@ class BigQueryQueryLogConfig:
     fetch_job_query_if_truncated: bool = True
 
 
-@dataclass
+@dataclass(config=ConnectorConfig)
 class BigQueryRunConfig(BaseConfig):
-    # Project ID to use
-    project_id: str
+    # List of project IDs to extract metadata from
+    project_ids: List[str]
 
     # Path to service account's JSON key file
     key_path: Optional[str] = None
@@ -80,9 +81,11 @@ class BigQueryRunConfig(BaseConfig):
     tag_matchers: List[TagMatcher] = field(default_factory=lambda: [])
 
     # configs for fetching query logs
-    query_log: BigQueryQueryLogConfig = BigQueryQueryLogConfig()
+    query_log: BigQueryQueryLogConfig = field(
+        default_factory=lambda: BigQueryQueryLogConfig()
+    )
 
-    @root_validator
-    def have_key_path_or_credentials(cls, values):
-        must_set_exactly_one(values, ["key_path", "credentials"])
-        return values
+    @model_validator(mode="after")
+    def have_key_path_or_credentials(self) -> "BigQueryRunConfig":
+        must_set_exactly_one(self.__dict__, ["key_path", "credentials"])
+        return self
