@@ -5,6 +5,7 @@ from typing import (
     Collection,
     Dict,
     Iterable,
+    Iterator,
     List,
     Mapping,
     Optional,
@@ -14,38 +15,43 @@ from typing import (
 
 try:
     import google.cloud.bigquery as bigquery
-    from google.cloud import logging_v2
 except ImportError:
     print("Please install metaphor[bigquery] extra\n")
     raise
 
 from metaphor.bigquery.config import BigQueryRunConfig
 from metaphor.bigquery.logEvent import JobChangeEvent
-from metaphor.bigquery.utils import LogEntry, build_client, build_logging_client
+from metaphor.bigquery.utils import (
+    BigQueryResource,
+    LogEntry,
+    build_client,
+    build_logging_client,
+    get_credentials,
+)
 from metaphor.common.base_extractor import BaseExtractor
-from metaphor.common.entity_id import dataset_normalized_name
+from metaphor.common.entity_id import dataset_normalized_name, to_dataset_entity_id
 from metaphor.common.event_util import ENTITY_TYPES
 from metaphor.common.fieldpath import FieldDataType, build_field_path
 from metaphor.common.filter import DatasetFilter
 from metaphor.common.logger import get_logger
-from metaphor.common.query_history import chunk_query_logs
+from metaphor.common.models import to_dataset_statistics
 from metaphor.common.tag_matcher import tag_datasets
-from metaphor.common.utils import md5_digest, start_of_day
+from metaphor.common.utils import md5_digest, safe_float, start_of_day
 from metaphor.models.crawler_run_metadata import Platform
 from metaphor.models.metadata_change_event import (
     DataPlatform,
     Dataset,
     DatasetLogicalID,
     DatasetSchema,
-    DatasetStatistics,
     DatasetStructure,
+    LogType,
     MaterializationType,
     QueriedDataset,
     QueryLog,
     SchemaField,
     SchemaType,
+    SourceInfo,
     SQLSchema,
-    TypeEnum,
 )
 
 logger = get_logger()
@@ -54,17 +60,19 @@ logger = get_logger()
 class BigQueryExtractor(BaseExtractor):
     """BigQuery metadata extractor"""
 
-    BYTES_PER_MEGABYTES = 1024 * 1024
+    _description = "BigQuery metadata crawler"
+    _platform = Platform.BIGQUERY
 
     @staticmethod
     def from_config_file(config_file: str) -> "BigQueryExtractor":
         return BigQueryExtractor(BigQueryRunConfig.from_yaml_file(config_file))
 
     def __init__(self, config: BigQueryRunConfig) -> None:
-        super().__init__(config, "BigQuery metadata crawler", Platform.BIGQUERY)
+        super().__init__(config)
         self._config = config
-        self._project_id = config.project_id
-        self._job_project_id = config.job_project_id
+        self._credentials = get_credentials(config)
+        self._project_ids = config.project_ids
+        self._job_project_id = config.job_project_id or self._credentials.project_id
         self._dataset_filter = config.filter.normalize()
         self._max_concurrency = config.max_concurrency
         self._tag_matchers = config.tag_matchers
@@ -78,11 +86,22 @@ class BigQueryExtractor(BaseExtractor):
             config.query_log.fetch_job_query_if_truncated
         )
 
-    async def extract(self) -> Collection[ENTITY_TYPES]:
-        logger.info(f"Fetching metadata from BigQuery project {self._project_id}")
+        self._datasets: List[Dataset] = []
 
-        client = build_client(self._config)
-        logging_client = build_logging_client(self._config)
+    async def extract(self) -> Collection[ENTITY_TYPES]:
+        for project_id in self._project_ids:
+            self._extract_project(project_id)
+
+        return self._datasets
+
+    def collect_query_logs(self) -> Iterator[QueryLog]:
+        for project_id in self._project_ids:
+            yield from self._fetch_query_logs(project_id)
+
+    def _extract_project(self, project_id: str) -> None:
+        logger.info(f"Fetching metadata from BigQuery project {project_id}")
+
+        client = build_client(project_id, self._credentials)
 
         fetched_tables: List[Dataset] = []
         for dataset_ref in BigQueryExtractor._list_datasets_with_filter(
@@ -124,13 +143,10 @@ class BigQueryExtractor(BaseExtractor):
             fetched_tables.extend(tables.values())
 
         logger.info("Fetching BigQueryAuditMetadata")
-        query_logs = self._fetch_query_logs(logging_client, client)
 
         tag_datasets(fetched_tables, self._tag_matchers)
 
-        entities: List[ENTITY_TYPES] = fetched_tables
-        entities.extend(chunk_query_logs(query_logs))
-        return entities
+        self._datasets.extend(fetched_tables)
 
     @staticmethod
     def _list_datasets_with_filter(
@@ -172,7 +188,9 @@ class BigQueryExtractor(BaseExtractor):
         )
 
         schema = BigQueryExtractor.parse_schema(bq_table)
-        statistics = BigQueryExtractor._parse_statistics(bq_table)
+        statistics = to_dataset_statistics(
+            bq_table.num_rows, bq_table.num_bytes, bq_table.modified
+        )
 
         return Dataset(
             logical_id=dataset_id,
@@ -181,6 +199,9 @@ class BigQueryExtractor(BaseExtractor):
             structure=DatasetStructure(
                 database=project_id, schema=bq_table.dataset_id, table=bq_table.table_id
             ),
+            source_info=SourceInfo(
+                created_at_source=bq_table.created, last_updated=bq_table.modified
+            ),
         )
 
     # See https://googleapis.dev/python/bigquery/latest/generated/google.cloud.bigquery.table.Table.html#google.cloud.bigquery.table.Table
@@ -235,9 +256,11 @@ class BigQueryExtractor(BaseExtractor):
             field_type = (
                 FieldDataType.ARRAY
                 if field.mode == "REPEATED"
-                else FieldDataType.RECORD
-                if field.field_type in ("RECORD", "STRUCT")
-                else FieldDataType.PRIMITIVE
+                else (
+                    FieldDataType.RECORD
+                    if field.field_type in ("RECORD", "STRUCT")
+                    else FieldDataType.PRIMITIVE
+                )
             )
 
             field_path = build_field_path(parent_field_path, field.name, field_type)
@@ -259,33 +282,22 @@ class BigQueryExtractor(BaseExtractor):
 
         return fields
 
-    # See https://googleapis.dev/python/bigquery/latest/generated/google.cloud.bigquery.table.Table.html#google.cloud.bigquery.table.Table
-    @staticmethod
-    def _parse_statistics(bq_table: bigquery.table.Table) -> DatasetStatistics:
-        return DatasetStatistics(
-            data_size=float(bq_table.num_bytes / BigQueryExtractor.BYTES_PER_MEGABYTES),
-            record_count=float(bq_table.num_rows),
-            last_updated=bq_table.modified,
-        )
-
-    def _fetch_query_logs(
-        self, logging_client: logging_v2.Client, client: bigquery.Client
-    ) -> List[QueryLog]:
-        logs: List[Optional[QueryLog]] = []
+    def _fetch_query_logs(self, project_id: str) -> Iterator[QueryLog]:
         log_filter = self._build_job_change_filter()
 
+        client = build_client(project_id, self._credentials)
+        logging_client = build_logging_client(project_id, self._credentials)
+        fetched_logs = 0
         for entry in logging_client.list_entries(
             page_size=self._query_log_fetch_size, filter_=log_filter
         ):
             if JobChangeEvent.can_parse(entry):
-                logs.append(self._parse_job_change_entry(entry, client))
+                log = self._parse_job_change_entry(entry, client)
+                if log:
+                    fetched_logs += 1
+                    yield log
 
-            if len(logs) % 1000 == 0:
-                logger.info(f"Fetched {len(logs)} audit logs")
-
-        logger.info(f"Number of audit log entries fetched: {len(logs)}")
-
-        return [log for log in logs if log is not None]
+        logger.info(f"Number of audit log entries fetched: {fetched_logs}")
 
     @staticmethod
     def _fetch_job_query(client: bigquery.Client, job_name: str) -> Optional[str]:
@@ -318,20 +330,12 @@ class BigQueryExtractor(BaseExtractor):
             return None
 
         sources: List[QueriedDataset] = [
-            QueriedDataset(database=d.project_id, schema=d.dataset_id, table=d.table_id)
+            self._convert_resource_to_queried_dataset(d)
             for d in job_change.source_tables
         ]
         target = job_change.destination_table
         target_datasets = (
-            [
-                QueriedDataset(
-                    database=target.project_id,
-                    schema=target.dataset_id,
-                    table=target.table_id,
-                )
-            ]
-            if target
-            else None
+            [self._convert_resource_to_queried_dataset(target)] if target else None
         )
 
         default_database, default_schema = None, None
@@ -353,60 +357,82 @@ class BigQueryExtractor(BaseExtractor):
             else None
         )
 
+        # Assume service accounts always end in ".gserviceaccount.com"
+        # https://cloud.google.com/compute/docs/access/service-accounts
+        is_service_account = job_change.user_email.endswith(".gserviceaccount.com")
+
         return QueryLog(
             id=f"{DataPlatform.BIGQUERY.name}:{job_change.job_name}",
             query_id=job_change.job_name,
             platform=DataPlatform.BIGQUERY,
             start_time=job_change.start_time,
-            duration=float(elapsed_time) if elapsed_time else None,
-            email=job_change.user_email,
-            rows_written=float(job_change.output_rows)
-            if job_change.output_rows
-            else None,
-            bytes_read=float(job_change.input_bytes)
-            if job_change.input_bytes
-            else None,
-            bytes_written=float(job_change.output_bytes)
-            if job_change.output_bytes
-            else None,
+            duration=safe_float(elapsed_time),
+            user_id=job_change.user_email if is_service_account else None,
+            email=job_change.user_email if not is_service_account else None,
+            rows_written=(
+                float(job_change.output_rows) if job_change.output_rows else None
+            ),
+            bytes_read=(
+                float(job_change.input_bytes) if job_change.input_bytes else None
+            ),
+            bytes_written=(
+                float(job_change.output_bytes) if job_change.output_bytes else None
+            ),
             sources=sources,
             targets=target_datasets,
             default_database=default_database,
             default_schema=default_schema,
-            type=self._map_query_type(job_change.statementType)
-            if job_change.statementType
-            else None,
+            type=(
+                self._map_query_type(job_change.statementType)
+                if job_change.statementType
+                else None
+            ),
             sql=query,
             sql_hash=md5_digest(job_change.query.encode("utf-8")),
         )
 
     # https://cloud.google.com/bigquery/docs/reference/auditlogs/rest/Shared.Types/BigQueryAuditMetadata.QueryStatementType
     _query_type_map = {
-        "SELECT": TypeEnum.SELECT,
-        "INSERT": TypeEnum.INSERT,
-        "UPDATE": TypeEnum.UPDATE,
-        "DELETE": TypeEnum.DELETE,
-        "MERGE": TypeEnum.MERGE,
-        "CREATE_TABLE": TypeEnum.CREATE_TABLE,
-        "CREATE_TABLE_AS_SELECT": TypeEnum.CREATE_TABLE,
-        "CREATE_SNAPSHOT_TABLE": TypeEnum.CREATE_TABLE,
-        "CREATE_VIEW": TypeEnum.CREATE_VIEW,
-        "CREATE_MATERIALIZED_VIEW": TypeEnum.CREATE_VIEW,
-        "DROP_TABLE": TypeEnum.DROP_TABLE,
-        "DROP_EXTERNAL_TABLE": TypeEnum.DROP_TABLE,
-        "DROP_SNAPSHOT_TABLE": TypeEnum.DROP_TABLE,
-        "DROP_VIEW": TypeEnum.DROP_VIEW,
-        "DROP_MATERIALIZED_VIEW": TypeEnum.DROP_VIEW,
-        "ALTER_TABLE": TypeEnum.ALTER_TABLE,
-        "ALTER_VIEW": TypeEnum.ALTER_VIEW,
-        "ALTER_MATERIALIZED_VIEW": TypeEnum.ALTER_VIEW,
-        "TRUNCATE_TABLE": TypeEnum.TRUNCATE,
-        "EXPORT_DATA": TypeEnum.EXPORT,
+        "SELECT": LogType.SELECT,
+        "INSERT": LogType.INSERT,
+        "UPDATE": LogType.UPDATE,
+        "DELETE": LogType.DELETE,
+        "MERGE": LogType.MERGE,
+        "CREATE_TABLE": LogType.CREATE_TABLE,
+        "CREATE_TABLE_AS_SELECT": LogType.CREATE_TABLE,
+        "CREATE_SNAPSHOT_TABLE": LogType.CREATE_TABLE,
+        "CREATE_VIEW": LogType.CREATE_VIEW,
+        "CREATE_MATERIALIZED_VIEW": LogType.CREATE_VIEW,
+        "DROP_TABLE": LogType.DROP_TABLE,
+        "DROP_EXTERNAL_TABLE": LogType.DROP_TABLE,
+        "DROP_SNAPSHOT_TABLE": LogType.DROP_TABLE,
+        "DROP_VIEW": LogType.DROP_VIEW,
+        "DROP_MATERIALIZED_VIEW": LogType.DROP_VIEW,
+        "ALTER_TABLE": LogType.ALTER_TABLE,
+        "ALTER_VIEW": LogType.ALTER_VIEW,
+        "ALTER_MATERIALIZED_VIEW": LogType.ALTER_VIEW,
+        "TRUNCATE_TABLE": LogType.TRUNCATE,
+        "EXPORT_DATA": LogType.EXPORT,
     }
 
     @staticmethod
-    def _map_query_type(query_type: str) -> TypeEnum:
-        return BigQueryExtractor._query_type_map.get(query_type.upper(), TypeEnum.OTHER)
+    def _convert_resource_to_queried_dataset(
+        resource: BigQueryResource,
+    ) -> QueriedDataset:
+        dataset_name = dataset_normalized_name(
+            resource.project_id, resource.dataset_id, resource.table_id
+        )
+        dataset_id = str(to_dataset_entity_id(dataset_name, DataPlatform.BIGQUERY))
+        return QueriedDataset(
+            id=dataset_id,
+            database=resource.project_id,
+            schema=resource.dataset_id,
+            table=resource.table_id,
+        )
+
+    @staticmethod
+    def _map_query_type(query_type: str) -> LogType:
+        return BigQueryExtractor._query_type_map.get(query_type.upper(), LogType.OTHER)
 
     def _build_job_change_filter(self) -> str:
         start_time = start_of_day(self._query_log_lookback_days).isoformat()
