@@ -12,7 +12,12 @@ from sql_metadata import Parser
 
 from metaphor.bigquery.lineage.config import BigQueryLineageRunConfig
 from metaphor.bigquery.logEvent import JobChangeEvent
-from metaphor.bigquery.utils import LogEntry, build_client, build_logging_client
+from metaphor.bigquery.utils import (
+    LogEntry,
+    build_client,
+    build_logging_client,
+    get_credentials,
+)
 from metaphor.common.base_extractor import BaseExtractor
 from metaphor.common.entity_id import dataset_normalized_name, to_dataset_entity_id
 from metaphor.common.event_util import ENTITY_TYPES
@@ -23,8 +28,7 @@ from metaphor.models.metadata_change_event import (
     DataPlatform,
     Dataset,
     DatasetLogicalID,
-    DatasetUpstream,
-    EntityType,
+    EntityUpstream,
 )
 
 logger = get_logger()
@@ -33,6 +37,9 @@ logger = get_logger()
 class BigQueryLineageExtractor(BaseExtractor):
     """BigQuery lineage metadata extractor"""
 
+    _description = "BigQuery data lineage crawler"
+    _platform = Platform.BIGQUERY
+
     @staticmethod
     def from_config_file(config_file: str) -> "BigQueryLineageExtractor":
         return BigQueryLineageExtractor(
@@ -40,9 +47,10 @@ class BigQueryLineageExtractor(BaseExtractor):
         )
 
     def __init__(self, config: BigQueryLineageRunConfig):
-        super().__init__(config, "BigQuery data lineage crawler", Platform.BIGQUERY)
+        super().__init__(config)
         self._config = config
-        self._project_id = config.project_id
+        self._credentials = get_credentials(config)
+        self._project_ids = config.project_ids
         self._dataset_filter = config.filter.normalize()
         self._enable_view_lineage = config.enable_view_lineage
         self._enable_lineage_from_log = config.enable_lineage_from_log
@@ -53,23 +61,29 @@ class BigQueryLineageExtractor(BaseExtractor):
         self._datasets: Dict[str, Dataset] = {}
 
     async def extract(self) -> Collection[ENTITY_TYPES]:
-        client = build_client(self._config)
-        logging_client = build_logging_client(self._config)
+        for project_id in self._project_ids:
+            self._extract_project(project_id)
+
+        return self._datasets.values()
+
+    def _extract_project(self, project_id: str):
+        logger.info(f"Extracting lineage for project {project_id}")
+
+        client = build_client(project_id, self._credentials)
+        logging_client = build_logging_client(project_id, self._credentials)
 
         if self._enable_view_lineage:
-            self._fetch_view_upstream(client)
+            self._fetch_view_upstream(client, project_id)
 
         if self._enable_lineage_from_log:
             self._fetch_audit_log(logging_client)
 
-        return self._datasets.values()
-
-    def _fetch_view_upstream(self, client: bigquery.Client) -> None:
+    def _fetch_view_upstream(self, client: bigquery.Client, project_id: str) -> None:
         logger.info("Fetching lineage info from BigQuery API")
 
         for bq_dataset in client.list_datasets():
             if not self._dataset_filter.include_schema(
-                self._project_id, bq_dataset.dataset_id
+                project_id, bq_dataset.dataset_id
             ):
                 logger.info(f"Skipped dataset {bq_dataset.dataset_id}")
                 continue
@@ -83,14 +97,14 @@ class BigQueryLineageExtractor(BaseExtractor):
             for bq_table in client.list_tables(bq_dataset.dataset_id):
                 table_ref = dataset_ref.table(bq_table.table_id)
                 if not self._dataset_filter.include_table(
-                    self._project_id, bq_dataset.dataset_id, bq_table.table_id
+                    project_id, bq_dataset.dataset_id, bq_table.table_id
                 ):
                     logger.info(f"Skipped table: {table_ref}")
                     continue
                 logger.debug(f"Found table {table_ref}")
 
-                bq_table = client.get_table(table_ref)
                 try:
+                    bq_table = client.get_table(table_ref)
                     self._parse_view_lineage(client.project, bq_table)
                 except Exception as ex:
                     logger.exception(ex)
@@ -131,8 +145,8 @@ class BigQueryLineageExtractor(BaseExtractor):
 
         if dataset_ids:
             dataset = self._init_dataset(view_name)
-            dataset.upstream = DatasetUpstream(
-                source_datasets=list(dataset_ids), transformation=view_query
+            dataset.entity_upstream = EntityUpstream(
+                source_entities=list(dataset_ids), transformation=view_query
             )
 
     def _fetch_audit_log(self, logging_client: logging_v2.Client):
@@ -180,11 +194,12 @@ class BigQueryLineageExtractor(BaseExtractor):
 
         table_name = destination.table_name()
         dataset = self._init_dataset(table_name)
-        dataset.upstream = DatasetUpstream(
-            source_datasets=[
-                str(to_dataset_entity_id(source.table_name(), DataPlatform.BIGQUERY))
-                for source in job_change.source_tables
-            ],
+        source_entities = [
+            str(to_dataset_entity_id(source.table_name(), DataPlatform.BIGQUERY))
+            for source in job_change.source_tables
+        ]
+        dataset.entity_upstream = EntityUpstream(
+            source_entities=source_entities,
             transformation=job_change.query,
         )
 
@@ -208,7 +223,6 @@ class BigQueryLineageExtractor(BaseExtractor):
     def _init_dataset(self, table_name: str) -> Dataset:
         if table_name not in self._datasets:
             self._datasets[table_name] = Dataset(
-                entity_type=EntityType.DATASET,
                 logical_id=DatasetLogicalID(
                     name=table_name, platform=DataPlatform.BIGQUERY
                 ),
