@@ -4,10 +4,11 @@ from pydantic.dataclasses import dataclass
 
 from metaphor.bigquery.config import BigQueryRunConfig
 from metaphor.common.column_statistics import ColumnStatistics
+from metaphor.common.dataclass import ConnectorConfig
 from metaphor.common.sampling import SamplingConfig
 
 
-@dataclass
+@dataclass(config=ConnectorConfig)
 class BigQueryProfileRunConfig(BigQueryRunConfig):
     # Compute specific types of statistics for each column
     column_statistics: ColumnStatistics = field(
