@@ -3,8 +3,6 @@ from functools import partial
 from time import sleep
 from typing import Collection, List, Set, Union
 
-from metaphor.common.entity_id import dataset_normalized_name
-
 try:
     import google.cloud.bigquery as bigquery
     from google.cloud.bigquery import QueryJob, TableReference
@@ -12,13 +10,17 @@ except ImportError:
     print("Please install metaphor[bigquery] extra\n")
     raise
 
-from metaphor.bigquery.extractor import BigQueryExtractor, build_client
+
+from metaphor.bigquery.extractor import BigQueryExtractor
 from metaphor.bigquery.profile.config import BigQueryProfileRunConfig, SamplingConfig
+from metaphor.bigquery.utils import build_client, get_credentials
 from metaphor.common.base_extractor import BaseExtractor
 from metaphor.common.column_statistics import ColumnStatistics
+from metaphor.common.entity_id import dataset_normalized_name
 from metaphor.common.event_util import ENTITY_TYPES
 from metaphor.common.filter import DatasetFilter
 from metaphor.common.logger import get_logger
+from metaphor.common.utils import safe_float
 from metaphor.models.crawler_run_metadata import Platform
 from metaphor.models.metadata_change_event import (
     DataPlatform,
@@ -26,7 +28,6 @@ from metaphor.models.metadata_change_event import (
     DatasetFieldStatistics,
     DatasetLogicalID,
     DatasetSchema,
-    EntityType,
     FieldStatistics,
 )
 
@@ -36,6 +37,9 @@ logger = get_logger()
 class BigQueryProfileExtractor(BaseExtractor):
     """BigQuery data profile extractor"""
 
+    _description = "BigQuery data profile crawler"
+    _platform = Platform.BIGQUERY
+
     @staticmethod
     def from_config_file(config_file: str) -> "BigQueryProfileExtractor":
         return BigQueryProfileExtractor(
@@ -43,20 +47,29 @@ class BigQueryProfileExtractor(BaseExtractor):
         )
 
     def __init__(self, config: BigQueryProfileRunConfig):
-        super().__init__(config, "BigQuery data profile crawler", Platform.BIGQUERY)
+        super().__init__(config)
         self._config = config
-        self._job_project_id = config.job_project_id
+        self._credentials = get_credentials(config)
+        self._project_ids = config.project_ids
+        self._job_project_id = config.job_project_id or self._credentials.project_id
         self._max_concurrency = config.max_concurrency
         self._filter = DatasetFilter.normalize(config.filter)
         self._column_statistics = config.column_statistics
         self._sampling = config.sampling
+        self._datasets: List[Dataset] = []
 
     async def extract(self) -> Collection[ENTITY_TYPES]:
-        logger.info("Fetching usage info from BigQuery")
+        for project_id in self._project_ids:
+            self._extract_project(project_id)
 
-        self._client = build_client(self._config)
+        return self._datasets
+
+    def _extract_project(self, project_id: str):
+        logger.info(f"Fetching usage info from BigQuery for project {project_id}")
+
+        self._client = build_client(project_id, self._credentials)
         tables = self._fetch_tables()
-        return self.profile(tables)
+        self.profile(tables)
 
     def _fetch_tables(self) -> List[TableReference]:
         tables = []
@@ -79,12 +92,10 @@ class BigQueryProfileExtractor(BaseExtractor):
                 logger.info(f"Found table {table_ref}")
 
                 tables.append(table_ref)
+
         return tables
 
-    def profile(
-        self,
-        tables: List[TableReference],
-    ) -> List[Dataset]:
+    def profile(self, tables: List[TableReference]):
         jobs: Set[QueryJob] = set()
 
         def profile(table: TableReference):
@@ -103,7 +114,7 @@ class BigQueryProfileExtractor(BaseExtractor):
             counter += 1
             sleep(1)
 
-        return datasets
+        self._datasets.extend(datasets)
 
     def _profile_table(
         self,
@@ -238,23 +249,19 @@ class BigQueryProfileExtractor(BaseExtractor):
             min_value, max_value, avg, std_dev = None, None, None, None
             if BigQueryProfileExtractor._is_numeric(data_type):
                 if column_statistics.min_value:
-                    min_value = (
-                        None if results[index] is None else float(results[index])
-                    )
+                    min_value = safe_float(results[index])
                     index += 1
 
                 if column_statistics.max_value:
-                    max_value = (
-                        None if results[index] is None else float(results[index])
-                    )
+                    max_value = safe_float(results[index])
                     index += 1
 
                 if column_statistics.avg_value:
-                    avg = None if results[index] is None else float(results[index])
+                    avg = safe_float(results[index])
                     index += 1
 
                 if column_statistics.std_dev:
-                    std_dev = None if results[index] is None else float(results[index])
+                    std_dev = safe_float(results[index])
                     index += 1
 
             fields.append(
@@ -304,7 +311,6 @@ class BigQueryProfileExtractor(BaseExtractor):
     @staticmethod
     def _init_dataset(full_name: str) -> Dataset:
         dataset = Dataset()
-        dataset.entity_type = EntityType.DATASET
         dataset.logical_id = DatasetLogicalID(
             name=full_name, platform=DataPlatform.BIGQUERY
         )
