@@ -1,7 +1,9 @@
 from pydantic.dataclasses import dataclass
 
+from metaphor.common.dataclass import ConnectorConfig
 
-@dataclass
+
+@dataclass(config=ConnectorConfig)
 class ColumnStatistics:
     # Compute null and non-null counts
     null_count: bool = True
@@ -20,3 +22,7 @@ class ColumnStatistics:
 
     # Compute standard deviation
     std_dev: bool = False
+
+    @property
+    def should_calculate(self) -> bool:
+        return any(bool(value) for value in self.__dict__.values())
