@@ -3,30 +3,32 @@ from typing import Optional, Union
 from canonicaljson import encode_canonical_json
 from pydantic.dataclasses import dataclass
 
+from metaphor.common.dataclass import ConnectorConfig
 from metaphor.common.event_util import EventUtil
 from metaphor.common.utils import md5_digest
+from metaphor.models.logical_id import PersonLogicalID
 from metaphor.models.metadata_change_event import (
     DashboardLogicalID,
     DataPlatform,
     DatasetLogicalID,
     EntityType,
-    GroupID,
     KnowledgeCardLogicalID,
-    PersonLogicalID,
+    PipelineLogicalID,
+    PipelineType,
     VirtualViewLogicalID,
     VirtualViewType,
 )
 
 
-@dataclass
+@dataclass(config=ConnectorConfig)
 class EntityId:
     type: EntityType
     logicalId: Union[
         DatasetLogicalID,
         DashboardLogicalID,
-        GroupID,
         KnowledgeCardLogicalID,
         PersonLogicalID,
+        PipelineLogicalID,
         VirtualViewLogicalID,
     ]
 
@@ -57,6 +59,13 @@ def to_dataset_entity_id_from_logical_id(logical_id: DatasetLogicalID) -> Entity
     return EntityId(EntityType.DATASET, logical_id)
 
 
+def to_pipeline_entity_id_from_logical_id(logical_id: PipelineLogicalID) -> EntityId:
+    """
+    converts a pipeline logical ID to entity ID
+    """
+    return EntityId(EntityType.PIPELINE, logical_id)
+
+
 def to_virtual_view_entity_id(name: str, virtualViewType: VirtualViewType) -> EntityId:
     """
     converts a virtual view name and type into a Virtual View entity ID
@@ -77,6 +86,23 @@ def to_person_entity_id(email: str) -> EntityId:
     )
 
 
+def to_pipeline_entity_id(name: str, pipeline_type: PipelineType) -> EntityId:
+    """
+    converts a pipeline name and type into a Pipeline entity ID
+    """
+    return EntityId(
+        EntityType.PIPELINE,
+        PipelineLogicalID(name=name, type=pipeline_type),
+    )
+
+
+def to_dashboard_entity_id_from_logical_id(logical_id: DashboardLogicalID) -> EntityId:
+    """
+    converts a dashboard logical ID to entity ID
+    """
+    return EntityId(EntityType.DASHBOARD, logical_id)
+
+
 def normalize_full_dataset_name(name: str) -> str:
     """
     Normalizes a fully qualified dataset name
