@@ -1,19 +1,24 @@
 import json
 import logging
 from importlib import resources
-from typing import Union
+from typing import Optional, Union
 
-import fastjsonschema
+from jsonschema import ValidationError
+from jsonschema.validators import validator_for
 
 from metaphor import models  # type: ignore
 from metaphor.models.metadata_change_event import (
     Dashboard,
     Dataset,
+    ExternalSearchDocument,
+    Hierarchy,
     KnowledgeCard,
     MetadataChangeEvent,
     Metric,
-    Person,
+    Pipeline,
+    QueryAttributions,
     QueryLogs,
+    UserActivity,
     VirtualView,
 )
 
@@ -21,52 +26,72 @@ logger = logging.getLogger(__name__)
 logger.setLevel(logging.INFO)
 
 ENTITY_TYPES = Union[
-    Dashboard, Dataset, Metric, Person, KnowledgeCard, VirtualView, QueryLogs
+    Dashboard,
+    Dataset,
+    ExternalSearchDocument,
+    Hierarchy,
+    KnowledgeCard,
+    Metric,
+    Pipeline,
+    QueryAttributions,
+    QueryLogs,
+    UserActivity,
+    VirtualView,
 ]
 
 
 class EventUtil:
     """Event utilities"""
 
-    with resources.open_text(models, "metadata_change_event.json") as f:
-        validate = fastjsonschema.compile(json.load(f))
+    def __init__(self):
+        with resources.open_text(models, "metadata_change_event.json") as f:
+            mce_schema = json.load(f)
 
-    def __init__(self, extractor_class="", server=""):
-        self._extractor_class = extractor_class
-        self._server = server
+        validator_class = validator_for(mce_schema)
+        validator_class.check_schema(mce_schema)
+        self._validator = validator_class(mce_schema)
 
-    def _build_event(self, **kwargs) -> MetadataChangeEvent:
+    @staticmethod
+    def _build_event(**kwargs) -> MetadataChangeEvent:
         """Create an MCE"""
         return MetadataChangeEvent(**kwargs)
 
-    def build_event(self, entity: ENTITY_TYPES):
+    @staticmethod
+    def build_event(entity: ENTITY_TYPES):
         """Build MCE given an entity"""
         if type(entity) is Dashboard:
-            return self._build_event(dashboard=entity)
+            return EventUtil._build_event(dashboard=entity)
         elif type(entity) is Dataset:
-            return self._build_event(dataset=entity)
+            return EventUtil._build_event(dataset=entity)
+        elif type(entity) is ExternalSearchDocument:
+            return EventUtil._build_event(external_search_document=entity)
         elif type(entity) is Metric:
-            return self._build_event(metric=entity)
-        elif type(entity) is Person:
-            return self._build_event(person=entity)
+            return EventUtil._build_event(metric=entity)
+        elif type(entity) is Pipeline:
+            return EventUtil._build_event(pipeline=entity)
         elif type(entity) is KnowledgeCard:
-            return self._build_event(knowledge_card=entity)
+            return EventUtil._build_event(knowledge_card=entity)
         elif type(entity) is VirtualView:
-            return self._build_event(virtual_view=entity)
+            return EventUtil._build_event(virtual_view=entity)
+        elif type(entity) is QueryAttributions:
+            return EventUtil._build_event(query_attributions=entity)
         elif type(entity) is QueryLogs:
-            return self._build_event(query_logs=entity)
+            return EventUtil._build_event(query_logs=entity)
+        elif type(entity) is Hierarchy:
+            return EventUtil._build_event(hierarchy=entity)
+        elif type(entity) is UserActivity:
+            return EventUtil._build_event(user_activity=entity)
         else:
             raise TypeError(f"invalid entity type {type(entity)}")
 
-    @staticmethod
-    def validate_message(message: dict) -> bool:
+    def validate_message(self, message: dict) -> Optional[dict]:
         """Validate message against json schema"""
         try:
-            EventUtil.validate(message)
-        except fastjsonschema.JsonSchemaException as e:
+            self._validator.validate(message)
+        except ValidationError as e:
             logger.error(f"MCE validation error: {e}. Message: {message}")
-            return False
-        return True
+            return None
+        return message
 
     @staticmethod
     def clean_nones(value):
@@ -90,6 +115,11 @@ class EventUtil:
         """Cast event to dict and remove all None values"""
         return EventUtil.clean_nones(event.to_dict())
 
+    @staticmethod
+    def build_then_trim(entity: ENTITY_TYPES) -> dict:
+        event = EventUtil.build_event(entity)
+        return EventUtil.trim_event(event)
+
     @staticmethod
     def class_fqcn(clazz) -> str:
         """Get the fully qualified class name"""
