@@ -4,13 +4,14 @@ from typing import Optional
 from urllib.parse import quote, urlparse
 
 from git import Repo
-from pydantic import root_validator
+from pydantic import model_validator
 from pydantic.dataclasses import dataclass
 
+from metaphor.common.dataclass import ConnectorConfig
 from metaphor.common.utils import must_set_exactly_one
 
 
-@dataclass
+@dataclass(config=ConnectorConfig)
 class GitRepoConfig:
     # Git repository URL, ending with ".git"
     git_url: str
@@ -27,10 +28,10 @@ class GitRepoConfig:
     # relative path to the project, default to the root of the repo
     project_path: str = ""
 
-    @root_validator
-    def have_token_or_password(cls, values):
-        must_set_exactly_one(values, ["access_token", "password"])
-        return values
+    @model_validator(mode="after")
+    def have_token_or_password(self) -> "GitRepoConfig":
+        must_set_exactly_one(self.__dict__, ["access_token", "password"])
+        return self
 
 
 def clone_repo(config: GitRepoConfig, local_dir: Optional[str] = None) -> str:
