@@ -1,5 +1,6 @@
 import logging
 from abc import ABC, abstractmethod
+from concurrent.futures import ThreadPoolExecutor
 from typing import Generator, List
 
 from metaphor.models.metadata_change_event import MetadataChangeEvent
@@ -13,10 +14,19 @@ logger.setLevel(logging.INFO)
 class Sink(ABC):
     """Base class for metadata sinks"""
 
-    def sink(self, events: List[MetadataChangeEvent]) -> bool:
+    def write_events(self, events: List[MetadataChangeEvent]) -> bool:
         """Sink MCE messages to the destination"""
-        records = [EventUtil.trim_event(e) for e in events]
-        valid_records = [r for r in records if EventUtil.validate_message(r)]
+        event_util = EventUtil()
+        records = [event_util.trim_event(e) for e in events]
+
+        logger.info("validating MCE records")
+        with ThreadPoolExecutor() as tpe:
+            valid_records = [
+                r
+                for r in tpe.map(event_util.validate_message, records)
+                if r is not None
+            ]
+
         if len(valid_records) == 0:
             return False
 
