@@ -3,13 +3,12 @@ from datetime import datetime
 from typing import Any, Collection, Dict, List
 
 import boto3
-from aws_assume_role_lib import assume_role
 
 from metaphor.common.base_extractor import BaseExtractor
 from metaphor.common.entity_id import dataset_normalized_name
 from metaphor.common.event_util import ENTITY_TYPES
 from metaphor.common.logger import get_logger
-from metaphor.common.utils import unique_list
+from metaphor.common.utils import safe_float, unique_list
 from metaphor.glue.config import AwsCredentials, GlueRunConfig
 from metaphor.models.crawler_run_metadata import Platform
 from metaphor.models.metadata_change_event import (
@@ -31,28 +30,22 @@ logger = get_logger()
 
 
 def create_glue_client(aws: AwsCredentials) -> boto3.client:
-    session = boto3.Session(
-        aws_access_key_id=aws.access_key_id,
-        aws_secret_access_key=aws.secret_access_key,
-        region_name=aws.region_name,
-    )
-    if aws.assume_role_arn is not None:
-        session = assume_role(session, aws.assume_role_arn)
-        logger.info(f"Assumed role: {session.client('sts').get_caller_identity()}")
-    return session.client("glue")
+    return aws.get_session().client("glue")
 
 
 class GlueExtractor(BaseExtractor):
     """Glue metadata extractor"""
 
     BYTES_PER_MEGABYTES = 1024 * 1024
+    _description = "Glue metadata crawler"
+    _platform = Platform.GLUE
 
     @staticmethod
     def from_config_file(config_file: str) -> "GlueExtractor":
         return GlueExtractor(GlueRunConfig.from_yaml_file(config_file))
 
     def __init__(self, config: GlueRunConfig) -> None:
-        super().__init__(config, "Glue metadata crawler", Platform.GLUE)
+        super().__init__(config)
         self._datasets: Dict[str, Dataset] = {}
         self._aws_config = config.aws
 
@@ -164,13 +157,15 @@ class GlueExtractor(BaseExtractor):
         dataset.schema.sql_schema.materialization = (
             MaterializationType.VIEW
             if table_type == "VIEW"
-            else MaterializationType.EXTERNAL
-            if table_type == "EXTERNAL"
-            else MaterializationType.TABLE
+            else (
+                MaterializationType.EXTERNAL
+                if table_type == "EXTERNAL"
+                else MaterializationType.TABLE
+            )
         )
 
         dataset.statistics = DatasetStatistics()
-        dataset.statistics.record_count = float(row_count) if row_count else None
+        dataset.statistics.record_count = safe_float(row_count)
         dataset.statistics.last_updated = last_updated
 
         dataset.structure = DatasetStructure(schema=schema, table=name)
