@@ -1,15 +1,16 @@
 from typing import Dict, Optional
 
-from pydantic import root_validator
+from pydantic import model_validator
 from pydantic.dataclasses import dataclass
 
 from metaphor.common.base_config import BaseConfig
+from metaphor.common.dataclass import ConnectorConfig
 from metaphor.common.git import GitRepoConfig
 from metaphor.common.utils import must_set_exactly_one
 from metaphor.models.metadata_change_event import DataPlatform
 
 
-@dataclass
+@dataclass(config=ConnectorConfig)
 class LookerConnectionConfig:
     # Database for the connection
     database: str
@@ -25,7 +26,7 @@ class LookerConnectionConfig:
     platform: DataPlatform = DataPlatform.SNOWFLAKE
 
 
-@dataclass
+@dataclass(config=ConnectorConfig)
 class LookerRunConfig(BaseConfig):
     base_url: str
     client_id: str
@@ -46,7 +47,7 @@ class LookerRunConfig(BaseConfig):
     verify_ssl: bool = True
     timeout: int = 120
 
-    @root_validator
-    def have_local_or_git_dir_for_lookml(cls, values):
-        must_set_exactly_one(values, ["lookml_dir", "lookml_git_repo"])
-        return values
+    @model_validator(mode="after")
+    def have_local_or_git_dir_for_lookml(self):
+        must_set_exactly_one(self.__dict__, ["lookml_dir", "lookml_git_repo"])
+        return self
