@@ -18,13 +18,14 @@ from metaphor.common.logger import get_logger
 from metaphor.looker.config import LookerConnectionConfig, LookerRunConfig
 from metaphor.looker.lookml_parser import Model, fullname, parse_project
 from metaphor.models.metadata_change_event import (
+    AssetStructure,
     Chart,
     ChartType,
     Dashboard,
     DashboardInfo,
     DashboardLogicalID,
     DashboardPlatform,
-    DashboardUpstream,
+    EntityUpstream,
     SourceInfo,
     VirtualViewType,
 )
@@ -35,6 +36,9 @@ logger = get_logger()
 class LookerExtractor(BaseExtractor):
     """Looker metadata extractor"""
 
+    _description = "Looker metadata crawler"
+    _platform = Platform.LOOKER
+
     vis_type_map = {
         "looker_area": ChartType.AREA,
         "looker_bar": ChartType.BAR,
@@ -59,7 +63,7 @@ class LookerExtractor(BaseExtractor):
         return LookerExtractor(LookerRunConfig.from_yaml_file(config_file))
 
     def __init__(self, config: LookerRunConfig) -> None:
-        super().__init__(config, "Looker metadata crawler", Platform.LOOKER)
+        super().__init__(config)
         self._base_url = config.base_url
         self._connections = config.connections
         self._lookml_dir = config.lookml_dir
@@ -128,12 +132,18 @@ class LookerExtractor(BaseExtractor):
             if dashboard.view_count is not None:
                 dashboard_info.view_count = float(dashboard.view_count)
 
-            upstream = None
+            entity_upstream = None
             if dashboard.dashboard_elements is not None:
-                (dashboard_info.charts, upstream) = self._extract_charts(
+                (dashboard_info.charts, entity_upstream) = self._extract_charts(
                     dashboard.dashboard_elements, model_map
                 )
 
+            assert dashboard.id is not None
+
+            # Dashboard id is guaranteed to look like `model_name::dashboard_name`
+            # Ref: https://www.googlecloudcommunity.com/gc/Technical-Tips-Tricks/How-can-I-find-the-id-of-a-LookML-dashboard/ta-p/592288
+            directory, name = dashboard.id.rsplit("::", 1)
+
             dashboards.append(
                 Dashboard(
                     logical_id=DashboardLogicalID(
@@ -141,7 +151,11 @@ class LookerExtractor(BaseExtractor):
                     ),
                     dashboard_info=dashboard_info,
                     source_info=source_info,
-                    upstream=upstream,
+                    entity_upstream=entity_upstream,
+                    structure=AssetStructure(
+                        directories=[directory],
+                        name=name,
+                    ),
                 )
             )
 
@@ -151,7 +165,7 @@ class LookerExtractor(BaseExtractor):
         self,
         dashboard_elements: Sequence[DashboardElement],
         model_map: Dict[str, Model],
-    ) -> Tuple[List[Chart], DashboardUpstream]:
+    ) -> Tuple[List[Chart], EntityUpstream]:
         charts = []
         explore_ids: Set[str] = set()
 
@@ -205,7 +219,7 @@ class LookerExtractor(BaseExtractor):
 
         return (
             charts,
-            DashboardUpstream(
-                source_virtual_views=list(explore_ids),
+            EntityUpstream(
+                source_entities=list(explore_ids),
             ),
         )
