@@ -3,15 +3,11 @@ import glob
 import logging
 import operator
 import os
-import re
 from dataclasses import dataclass
 from typing import Dict, List, Optional, Set, Tuple
 
-from metaphor.looker.config import LookerConnectionConfig
-
 try:
     import lkml
-    import sql_metadata
 except ImportError:
     print("Please install metaphor[looker] extra\n")
     raise
@@ -21,13 +17,22 @@ from metaphor.common.entity_id import (
     to_dataset_entity_id,
     to_virtual_view_entity_id,
 )
-from metaphor.common.logger import get_logger
+from metaphor.common.logger import get_logger, json_dump_to_debug_file
+from metaphor.common.utils import unique_list
+from metaphor.looker.config import LookerConnectionConfig
 from metaphor.models.metadata_change_event import (
+    AssetStructure,
+    EntityType,
+    EntityUpstream,
     LookerExplore,
     LookerExploreJoin,
     LookerView,
     LookerViewDimension,
     LookerViewMeasure,
+    LookerViewQuery,
+    SystemTag,
+    SystemTags,
+    SystemTagSource,
     VirtualView,
     VirtualViewLogicalID,
     VirtualViewType,
@@ -77,6 +82,23 @@ class Model:
         return Model(explores=dict((e.name, e) for e in explores))
 
 
+_parsed_files: Dict[str, Dict] = {}
+
+
+def _parse_lookml_file(path, base_dir) -> Dict:
+
+    if path in _parsed_files:
+        return _parsed_files[path]
+
+    with open(path) as f:
+        model = lkml.load(f)
+        _parsed_files[path] = model
+
+        sanitized = os.path.relpath(path, base_dir).replace("/", "__")
+        json_dump_to_debug_file(model, f"{sanitized}.json")
+        return model
+
+
 def _to_dataset_id(source_name: str, connection: LookerConnectionConfig) -> EntityId:
     parts = source_name.split(".")
 
@@ -102,90 +124,84 @@ def _to_dataset_id(source_name: str, connection: LookerConnectionConfig) -> Enti
     return to_dataset_entity_id(full_name, connection.platform, connection.account)
 
 
-def _get_upstream_datasets(
-    view_name, raw_model: RawModel, connection: LookerConnectionConfig
-) -> Set[EntityId]:
+def _get_upstream_and_query(
+    model_name: str,
+    view_name: str,
+    raw_model: RawModel,
+    connection: LookerConnectionConfig,
+) -> Tuple[Set[EntityId], Optional[LookerViewQuery]]:
     raw_views = raw_model.raw_views
     raw_view = raw_views.get(view_name)
     if raw_view is None:
         logger.error(f"Refer to non-existing view {view_name}")
-        return set()
+        return set(), None
 
-    if "upstream_dataset_ids" in raw_view:
-        return raw_view["upstream_dataset_ids"]
+    if "upstream_entity_ids" in raw_view:
+        return raw_view["upstream_entity_ids"], raw_view["query"]
 
-    upstreams = set()
+    upstreams = set()  # upstream dataset id or virtual view id
+    query = None
 
     # Set upstream via derived_table
     # https://docs.looker.com/reference/view-params/derived_table
-    derived_table = raw_view.get("derived_table", None)
+    derived_table: Optional[Dict] = raw_view.get("derived_table")
     if derived_table is not None:
         if "sql" in derived_table:
-            upstreams = set(
-                _extract_upstream_datasets_from_sql(
-                    derived_table["sql"], raw_model, connection
-                )
+            query = LookerViewQuery(
+                query=derived_table["sql"],
+                source_platform=connection.platform,
+                source_dataset_account=connection.account,
+                default_database=connection.database,
+                default_schema=connection.default_schema,
             )
 
         # https://docs.looker.com/data-modeling/learning-lookml/creating-ndts
         if "explore_source" in derived_table:
             explore_name = derived_table["explore_source"]["name"]
-            raw_explore = raw_model.raw_explores.get(explore_name, None)
+            raw_explore = raw_model.raw_explores.get(explore_name)
             assert raw_explore is not None, f"Invalid explore_source: {explore_name}"
 
             base_view_name = _get_base_view_name(raw_explore, raw_model)
             if base_view_name is None:
                 base_view_name = explore_name
 
-            upstreams.update(
-                _get_upstream_datasets(base_view_name, raw_model, connection)
+            assert base_view_name in raw_views, f"Invalid base view {base_view_name}"
+
+            # TODO: handle field mapping
+
+            upstreams.add(
+                to_virtual_view_entity_id(
+                    fullname(model_name, base_view_name), VirtualViewType.LOOKER_VIEW
+                )
             )
 
     # Set upstream via sql_table_name
     # https://docs.looker.com/reference/view-params/sql_table_name-for-view
-    sql_table_name = raw_view.get("sql_table_name", None)
+    sql_table_name: Optional[str] = raw_view.get("sql_table_name")
     if sql_table_name is not None:
-        upstreams = set([_to_dataset_id(sql_table_name, connection)])
+        upstreams = {_to_dataset_id(sql_table_name, connection)}
 
     # If none of the above was triggered, assume upstream is a table with the same name
     # https://docs.looker.com/reference/view-params/sql_table_name-for-view
-    if len(upstreams) == 0:
+    if query is None and len(upstreams) == 0:
         upstreams.add(_to_dataset_id(view_name, connection))
 
-    raw_view["upstream_dataset_ids"] = upstreams
-    return upstreams
-
-
-def _extract_upstream_datasets_from_sql(
-    sql: str, raw_model: RawModel, connection: LookerConnectionConfig
-) -> Set[EntityId]:
-    upstreams: Set[EntityId] = set()
-    try:
-        # strip the brackets around referenced view name
-        sql = re.sub(r"\${(.+\.SQL_TABLE_NAME)}", r"\1", sql)
-
-        # parse SQL tables
-        tables = sql_metadata.Parser(sql).tables
-        for table in tables:
-            if table.endswith(".SQL_TABLE_NAME"):
-                # Selecting from another derived table
-                # https://docs.looker.com/data-modeling/learning-lookml/sql-and-referring-to-lookml
-                view_name = table.split(".")[0]
-                upstreams.update(
-                    _get_upstream_datasets(view_name, raw_model, connection)
-                )
-            else:
-                upstreams.add(_to_dataset_id(table, connection))
-    except Exception as e:
-        logger.warning(f"Failed to parse SQL:\n{sql}\n\nError:{e}")
-
-    return upstreams
+    raw_view["upstream_entity_ids"] = upstreams
+    raw_view["query"] = query
+    return upstreams, query
 
 
 def fullname(model: str, name: str) -> str:
     return f"{model}.{name}"
 
 
+def _get_model_asset_structure(model: str, name: str) -> AssetStructure:
+    return AssetStructure(
+        directories=[model],
+        name=name,
+    )
+
+
 def _build_looker_view(
     model: str,
     raw_view: Dict,
@@ -199,12 +215,22 @@ def _build_looker_view(
         url=url,
     )
 
+    source_entities = []
     try:
-        view.source_datasets = [
-            str(ds) for ds in _get_upstream_datasets(name, raw_model, connection)
-        ]
+        upstream_entity_ids, query = _get_upstream_and_query(
+            model, name, raw_model, connection
+        )
+        if upstream_entity_ids:
+            source_entities = [str(entity_id) for entity_id in upstream_entity_ids]
+            view.source_datasets = [
+                str(entity_id)
+                for entity_id in upstream_entity_ids
+                if entity_id.type == EntityType.DATASET
+            ] or None
+        if query is not None:
+            view.query = query
     except Exception:
-        logger.exception(f"Can't determine upstream datasets for view {name}")
+        logger.exception(f"Can't determine upstream for view {name}")
 
     if "dimensions" in raw_view:
         view.dimensions = [
@@ -229,6 +255,10 @@ def _build_looker_view(
             name=fullname(model, name), type=VirtualViewType.LOOKER_VIEW
         ),
         looker_view=view,
+        structure=_get_model_asset_structure(model, name),
+        entity_upstream=(
+            EntityUpstream(source_entities=source_entities) if source_entities else None
+        ),
     )
 
 
@@ -250,12 +280,12 @@ def _build_looker_explore(
 ) -> VirtualView:
     name = raw_explore["name"]
     base_view_name = _get_base_view_name(raw_explore, raw_model)
+    tag_names: List[str] = raw_explore.get("tags", [])
 
     explore = LookerExplore(
         model_name=model,
         description=raw_explore.get("description"),
         label=raw_explore.get("label"),
-        tags=raw_explore.get("tags"),
         url=url,
         fields=raw_explore.get("fields"),
         base_view=str(
@@ -295,11 +325,23 @@ def _build_looker_explore(
 
     # TODO: combine access_filters, always_filters and conditional_filters into explore.filters
 
+    # combine base view and joined view into source entities
+    source_entities = unique_list(
+        [explore.base_view] + [join.view for join in explore.joins or []]
+    )
+
+    tags = [
+        SystemTag(key=None, system_tag_source=SystemTagSource.LOOKER, value=value)
+        for value in tag_names
+    ]
     return VirtualView(
         logical_id=VirtualViewLogicalID(
             name=fullname(model, name), type=VirtualViewType.LOOKER_EXPLORE
         ),
         looker_explore=explore,
+        structure=_get_model_asset_structure(model, name),
+        entity_upstream=EntityUpstream(source_entities=source_entities),
+        system_tags=SystemTags(tags=tags),
     )
 
 
@@ -473,33 +515,32 @@ def _load_included_file(
         processed_files.add(normpath)
 
         url = _get_entity_url(file_path, base_dir, projectSourceUrl)
-        with open(file_path) as f:
-            logger.info(f"Processing view file {normpath}")
+        logger.info(f"Processing view file {normpath}")
 
-            root = lkml.load(f)
-            for view in root.get("views", []):
-                raw_views[view["name"]] = view
-                entity_urls[view["name"]] = url
+        root = _parse_lookml_file(file_path, base_dir)
+        for view in root.get("views", []):
+            raw_views[view["name"]] = view
+            entity_urls[view["name"]] = url
 
-            for explore in root.get("explores", []):
-                raw_explores[explore["name"]] = explore
-                entity_urls[explore["name"]] = url
+        for explore in root.get("explores", []):
+            raw_explores[explore["name"]] = explore
+            entity_urls[explore["name"]] = url
 
-            # A file can further include other files
-            # https://docs.looker.com/reference/model-params/include#using_include_in_a_view_file
-            for include_path in root.get("includes", []):
-                # Convert to absolute include
-                include_path = _to_absolute_include(include_path, file_path, base_dir)
+        # A file can further include other files
+        # https://docs.looker.com/reference/model-params/include#using_include_in_a_view_file
+        for include_path in root.get("includes", []):
+            # Convert to absolute include
+            include_path = _to_absolute_include(include_path, file_path, base_dir)
 
-                _load_included_file(
-                    include_path,
-                    base_dir,
-                    projectSourceUrl,
-                    raw_views,
-                    raw_explores,
-                    entity_urls,
-                    processed_files,
-                )
+            _load_included_file(
+                include_path,
+                base_dir,
+                projectSourceUrl,
+                raw_views,
+                raw_explores,
+                entity_urls,
+                processed_files,
+            )
 
 
 def _load_model(
@@ -511,9 +552,7 @@ def _load_model(
     """
     Loads model file and extract raw Views and Explores
     """
-    with open(model_path) as f:
-        model = lkml.load(f)
-
+    model = _parse_lookml_file(model_path, base_dir)
     logger.info(f"Processing model {model_path}")
 
     raw_views: Dict[str, Dict] = {}
