@@ -1,33 +1,33 @@
 from datetime import datetime
 from typing import Dict, Optional
 
-from pydantic import BaseModel
+from metaphor.common.models import V1CompatBaseModel
 
 
-class MssqlConnectConfig(BaseModel):
+class MssqlConnectConfig(V1CompatBaseModel):
     endpoint: str
     username: str
     password: str
 
 
-class MssqlDatabase(BaseModel):
+class MssqlDatabase(V1CompatBaseModel):
     id: int
     name: str
     create_time: datetime
     collation_name: str
 
 
-class MssqlColumn(BaseModel):
+class MssqlColumn(V1CompatBaseModel):
     name: str
     type: str
     max_length: float
     precision: float
     is_nullable: bool
-    is_unique: Optional[bool]
-    is_primary_key: Optional[bool]
+    is_unique: Optional[bool] = None
+    is_primary_key: Optional[bool] = None
 
 
-class MssqlForeignKey(BaseModel):
+class MssqlForeignKey(V1CompatBaseModel):
     name: str
     table_id: str
     column_name: str
@@ -35,7 +35,7 @@ class MssqlForeignKey(BaseModel):
     referenced_column: str
 
 
-class MssqlTable(BaseModel):
+class MssqlTable(V1CompatBaseModel):
     id: str
     name: str
     schema_name: str
@@ -43,19 +43,19 @@ class MssqlTable(BaseModel):
     column_dict: Dict[str, MssqlColumn]
     create_time: datetime
     is_external: bool
-    external_source: Optional[str]
-    external_file_format: Optional[str]
+    external_source: Optional[str] = None
+    external_file_format: Optional[str] = None
 
 
-class MssqlQueryLog(BaseModel):
+class MssqlQueryLog(V1CompatBaseModel):
     request_id: str
-    session_id: Optional[str]
+    session_id: Optional[str] = None
     sql_query: str
     login_name: str
     start_time: datetime
     end_time: datetime
     duration_in_ms: int
-    query_size_in_mb: Optional[int]
-    error: Optional[str]
-    row_count: Optional[int]
-    query_operation: Optional[str]
+    query_size_in_mb: Optional[int] = None
+    error: Optional[str] = None
+    row_count: Optional[int] = None
+    query_operation: Optional[str] = None
