@@ -10,13 +10,13 @@ from metaphor.common.base_extractor import BaseExtractor
 from metaphor.common.entity_id import dataset_normalized_name
 from metaphor.common.event_util import ENTITY_TYPES
 from metaphor.common.logger import get_logger
+from metaphor.common.models import to_dataset_statistics
 from metaphor.models.crawler_run_metadata import Platform
 from metaphor.models.metadata_change_event import (
     DataPlatform,
     Dataset,
     DatasetLogicalID,
     DatasetSchema,
-    DatasetStatistics,
     DatasetStructure,
     ForeignKey,
     MaterializationType,
@@ -40,25 +40,22 @@ _ignored_schemas = [
 class PostgreSQLExtractor(BaseExtractor):
     """PostgreSQL metadata extractor"""
 
+    _description = "PostgreSQL metadata crawler"
+    _platform = Platform.POSTGRESQL
+    _dataset_platform = DataPlatform.POSTGRESQL
+
     @staticmethod
     def from_config_file(config_file: str) -> "PostgreSQLExtractor":
         return PostgreSQLExtractor(PostgreSQLRunConfig.from_yaml_file(config_file))
 
-    def __init__(
-        self,
-        config: PostgreSQLRunConfig,
-        description="PostgreSQL metadata crawler",
-        platform=Platform.POSTGRESQL,
-        dataset_platform=DataPlatform.POSTGRESQL,
-    ):
-        super().__init__(config, description, platform)
+    def __init__(self, config: PostgreSQLRunConfig):
+        super().__init__(config)
         self._host = config.host
         self._database = config.database
         self._user = config.user
         self._password = config.password
         self._filter = config.filter.normalize()
         self._port = config.port
-        self._dataset_platform = dataset_platform
 
         self._datasets: Dict[str, Dataset] = {}
 
@@ -340,16 +337,14 @@ class PostgreSQLExtractor(BaseExtractor):
         dataset.schema.sql_schema.materialization = (
             MaterializationType.VIEW
             if table_type == "VIEW"
-            else MaterializationType.EXTERNAL
-            if table_type == "EXTERNAL"
-            else MaterializationType.TABLE
+            else (
+                MaterializationType.EXTERNAL
+                if table_type == "EXTERNAL"
+                else MaterializationType.TABLE
+            )
         )
 
-        dataset.statistics = DatasetStatistics()
-        dataset.statistics.record_count = float(row_count) if row_count else None
-        dataset.statistics.data_size = (
-            table_size / (1000 * 1000) if table_size else None
-        )  # in MB
+        dataset.statistics = to_dataset_statistics(row_count, table_size)
         # There is no reliable way to directly get data last modified time, can explore alternatives in future
         # https://dba.stackexchange.com/questions/58214/getting-last-modification-date-of-a-postgresql-database-table/168752
 
