@@ -3,11 +3,12 @@ from typing import List, Set
 
 from pydantic.dataclasses import dataclass
 
+from metaphor.common.dataclass import ConnectorConfig
 from metaphor.common.tag_matcher import TagMatcher
 from metaphor.postgresql.config import PostgreSQLRunConfig
 
 
-@dataclass
+@dataclass(config=ConnectorConfig)
 class RedshiftQueryLogConfig:
     # Number of days back of query logs to fetch, if 0, don't fetch query logs
     lookback_days: int = 1
@@ -16,7 +17,7 @@ class RedshiftQueryLogConfig:
     excluded_usernames: Set[str] = field(default_factory=lambda: set())
 
 
-@dataclass
+@dataclass(config=ConnectorConfig)
 class RedshiftRunConfig(PostgreSQLRunConfig):
     port: int = 5439
 
@@ -24,4 +25,6 @@ class RedshiftRunConfig(PostgreSQLRunConfig):
     tag_matchers: List[TagMatcher] = field(default_factory=lambda: [])
 
     # configs for fetching query logs
-    query_log: RedshiftQueryLogConfig = RedshiftQueryLogConfig()
+    query_log: RedshiftQueryLogConfig = field(
+        default_factory=lambda: RedshiftQueryLogConfig()
+    )
