@@ -3,6 +3,7 @@ from metaphor.models.crawler_run_metadata import Platform
 from metaphor.models.metadata_change_event import DataPlatform
 from metaphor.postgresql.profile.extractor import PostgreSQLProfileExtractor
 from metaphor.redshift.profile.config import RedshiftProfileRunConfig
+from metaphor.redshift.utils import exclude_system_databases
 
 logger = get_logger()
 
@@ -10,6 +11,10 @@ logger = get_logger()
 class RedshiftProfileExtractor(PostgreSQLProfileExtractor):
     """Redshift data profile extractor"""
 
+    _description = "Redshift data profile crawler"
+    _platform = Platform.REDSHIFT
+    _dataset_platform = DataPlatform.REDSHIFT
+
     @staticmethod
     def from_config_file(config_file: str) -> "RedshiftProfileExtractor":
         return RedshiftProfileExtractor(
@@ -17,9 +22,5 @@ class RedshiftProfileExtractor(PostgreSQLProfileExtractor):
         )
 
     def __init__(self, config: RedshiftProfileRunConfig):
-        super().__init__(
-            config,
-            "Redshift data profile crawler",
-            Platform.REDSHIFT,
-            DataPlatform.REDSHIFT,
-        )
+        super().__init__(config)
+        self._filter = exclude_system_databases(self._filter)
