@@ -3,12 +3,13 @@ from dataclasses import field
 from pydantic.dataclasses import dataclass
 
 from metaphor.common.column_statistics import ColumnStatistics
+from metaphor.common.dataclass import ConnectorConfig
 from metaphor.common.sampling import SamplingConfig
-from metaphor.snowflake.config import SnowflakeRunConfig
+from metaphor.snowflake.config import SnowflakeBaseConfig
 
 
-@dataclass
-class SnowflakeProfileRunConfig(SnowflakeRunConfig):
+@dataclass(config=ConnectorConfig)
+class SnowflakeProfileRunConfig(SnowflakeBaseConfig):
     # Compute specific types of statistics for each column
     column_statistics: ColumnStatistics = field(
         default_factory=lambda: ColumnStatistics()
