@@ -1,7 +1,6 @@
 import logging
-from typing import Collection, Dict, List, Tuple
-
-from metaphor.models.crawler_run_metadata import Platform
+from collections import defaultdict
+from typing import Collection, Dict, List, NamedTuple, Tuple
 
 try:
     from snowflake.connector import ProgrammingError, SnowflakeConnection
@@ -9,17 +8,23 @@ except ImportError:
     print("Please install metaphor[snowflake] extra\n")
     raise
 
+
 from metaphor.common.base_extractor import BaseExtractor
-from metaphor.common.entity_id import dataset_normalized_name
+from metaphor.common.entity_id import (
+    dataset_normalized_name,
+    normalize_full_dataset_name,
+)
 from metaphor.common.event_util import ENTITY_TYPES
 from metaphor.common.logger import get_logger
 from metaphor.common.sampling import SamplingConfig
+from metaphor.common.snowflake import normalize_snowflake_account
+from metaphor.common.utils import safe_float, safe_int
+from metaphor.models.crawler_run_metadata import Platform
 from metaphor.models.metadata_change_event import (
     DataPlatform,
     Dataset,
     DatasetFieldStatistics,
     DatasetLogicalID,
-    EntityType,
     FieldStatistics,
 )
 from metaphor.snowflake import auth
@@ -28,21 +33,34 @@ from metaphor.snowflake.profile.config import (
     ColumnStatistics,
     SnowflakeProfileRunConfig,
 )
-from metaphor.snowflake.utils import (
-    DatasetInfo,
-    QueryWithParam,
-    SnowflakeTableType,
-    async_execute,
-)
+from metaphor.snowflake.utils import DatasetInfo, QueryWithParam, SnowflakeTableType
 
 logger = get_logger()
 
 logging.getLogger("snowflake.connector").setLevel(logging.WARNING)
 
 
+class Table(NamedTuple):
+    catalog: str
+    schema: str
+    name: str
+
+    @property
+    def full_name(self) -> str:
+        return f"{self.catalog}.{self.schema}.{self.name}"
+
+    @classmethod
+    def from_name(cls, name: str) -> "Table":
+        catalog, schema, table_name = name.split(".")
+        return Table(catalog, schema, table_name)
+
+
 class SnowflakeProfileExtractor(BaseExtractor):
     """Snowflake data profile extractor"""
 
+    _description = "Snowflake data profile crawler"
+    _platform = Platform.SNOWFLAKE
+
     @staticmethod
     def from_config_file(config_file: str) -> "SnowflakeProfileExtractor":
         return SnowflakeProfileExtractor(
@@ -50,8 +68,8 @@ class SnowflakeProfileExtractor(BaseExtractor):
         )
 
     def __init__(self, config: SnowflakeProfileRunConfig):
-        super().__init__(config, "Snowflake data profile crawler", Platform.SNOWFLAKE)
-        self._account = config.account
+        super().__init__(config)
+        self._account = normalize_snowflake_account(config.account)
         self._filter = config.filter.normalize().merge(DEFAULT_FILTER)
         self._max_concurrency = config.max_concurrency
         self._include_views = config.include_views
@@ -77,7 +95,9 @@ class SnowflakeProfileExtractor(BaseExtractor):
 
             for database in databases:
                 tables = self._fetch_tables(cursor, database)
-                logger.info(f"Include {len(tables)} tables from {database}")
+                logger.info(
+                    f"Include {len(tables)} {'tables/views' if self._include_views else 'tables'} from {database}"
+                )
 
                 self._fetch_columns_async(self._conn, tables)
 
@@ -112,7 +132,7 @@ class SnowflakeProfileExtractor(BaseExtractor):
                 self._account, normalized_name
             )
             tables[normalized_name] = DatasetInfo(
-                database, schema, name, table_type, int(row_count)
+                database, schema, name, table_type, safe_int(row_count)
             )
 
         return tables
@@ -129,57 +149,74 @@ class SnowflakeProfileExtractor(BaseExtractor):
         tables: Dict[str, DatasetInfo],
     ) -> None:
         # Create a map of full_name => [column_info] from information_schema.columns
+        column_info_map: Dict[Table, List[Tuple[str, str]]] = defaultdict(list)
+        profile_queries = {}
+
         cursor = connection.cursor()
-        cursor.execute(self.FETCH_COLUMNS_QUERY)
-        column_info_map: Dict[str, List[Tuple[str, str]]] = {}
+        cursor.execute("SHOW COLUMNS")
+
+        # This could include columns in views or stream tables.
         for row in cursor:
+            # See https://docs.snowflake.com/en/sql-reference/sql/show-columns#output
             (
-                table_catalog,
-                table_schema,
                 table_name,
+                table_schema,
                 column_name,
                 data_type,
+                _,
+                _,
+                _,
+                _,
+                _,
+                table_catalog,
+                *_,
             ) = row
-            normalized_name = dataset_normalized_name(
-                table_catalog, table_schema, table_name
-            )
-            if normalized_name not in tables:
-                continue
-
-            column_info_map.setdefault(normalized_name, []).append(
+            column_info_map[Table(table_catalog, table_schema, table_name)].append(
                 (column_name, data_type)
             )
 
-        # Build profile query for each table
-        profile_queries = {}
-        for normalized_name, column_info in column_info_map.items():
-            dataset_info = tables.get(normalized_name)
-            assert dataset_info is not None
+        for table, column_info in column_info_map.items():
+            row_count = None
+            dataset_info = tables.get(
+                normalize_full_dataset_name(table.full_name)
+            )  # FIXME normalized name is case insensitive, make it case sensitive so that this logic works properly
+            if dataset_info is None:
+                continue
 
-            profile_queries[normalized_name] = QueryWithParam(
+            row_count = dataset_info.row_count
+
+            profile_queries[table.full_name] = QueryWithParam(
                 SnowflakeProfileExtractor._build_profiling_query(
                     column_info,
-                    dataset_info.schema,
-                    dataset_info.name,
-                    dataset_info.row_count or 0,
+                    table.schema,
+                    table.name,
+                    row_count or 0,
                     self._column_statistics,
                     self._sampling,
                 )
             )
 
+        # Has to be here, otherwise patching during test would not work
+        from metaphor.snowflake.utils import async_execute
+
         profiles = async_execute(
             connection, profile_queries, "profile_columns", self._max_concurrency
         )
 
-        for normalized_name, profile in profiles.items():
-            dataset = self._datasets[normalized_name]
+        for full_name, profile in profiles.items():
+            # We need to make sure full_name points to an actual table,
+            # instead of view or stream table.
+            dataset = self._datasets.get(
+                normalize_full_dataset_name(full_name)
+            )  # FIXME normalized name is case insensitive, make it case sensitive so that this logic works properly
 
-            SnowflakeProfileExtractor._parse_profiling_result(
-                column_info_map[normalized_name],
-                profile[0],
-                dataset,
-                self._column_statistics,
-            )
+            if dataset:
+                SnowflakeProfileExtractor._parse_profiling_result(
+                    column_info_map[Table.from_name(full_name)],
+                    profile[0],
+                    dataset,
+                    self._column_statistics,
+                )
 
     @staticmethod
     def _build_profiling_query(
@@ -254,23 +291,19 @@ class SnowflakeProfileExtractor(BaseExtractor):
             min_value, max_value, avg, std_dev = None, None, None, None
             if SnowflakeProfileExtractor._is_numeric(data_type):
                 if column_statistics.min_value:
-                    min_value = (
-                        None if results[index] is None else float(results[index])
-                    )
+                    min_value = safe_float(results[index])
                     index += 1
 
                 if column_statistics.max_value:
-                    max_value = (
-                        None if results[index] is None else float(results[index])
-                    )
+                    max_value = safe_float(results[index])
                     index += 1
 
                 if column_statistics.avg_value:
-                    avg = None if results[index] is None else float(results[index])
+                    avg = safe_float(results[index])
                     index += 1
 
                 if column_statistics.std_dev:
-                    std_dev = None if results[index] is None else float(results[index])
+                    std_dev = safe_float(results[index])
                     index += 1
 
             fields.append(
@@ -303,7 +336,6 @@ class SnowflakeProfileExtractor(BaseExtractor):
     @staticmethod
     def _init_dataset(account: str, normalized_name: str) -> Dataset:
         dataset = Dataset()
-        dataset.entity_type = EntityType.DATASET
         dataset.logical_id = DatasetLogicalID(
             name=normalized_name, account=account, platform=DataPlatform.SNOWFLAKE
         )
