@@ -1,15 +1,14 @@
-from typing import Collection, Dict, Iterable, List, Optional
+from typing import Collection, Dict, Iterable, Iterator, List, Optional, Set
 
 from metaphor.common.event_util import ENTITY_TYPES
 from metaphor.common.logger import get_logger
-from metaphor.common.query_history import chunk_query_logs
 from metaphor.common.utils import generate_querylog_id, md5_digest, start_of_day
 from metaphor.models.crawler_run_metadata import Platform
 from metaphor.models.metadata_change_event import (
     DataPlatform,
+    LogType,
     Parsing,
     QueryLog,
-    TypeEnum,
 )
 from metaphor.mssql.extractor import MssqlExtractor
 from metaphor.mssql.mssql_client import MssqlClient
@@ -23,93 +22,109 @@ logger = get_logger()
 class SynapseExtractor(MssqlExtractor):
     """Synapse metadata extractor"""
 
+    _description = "Synapse metadata crawler"
+    _platform = Platform.SYNAPSE
+    _dataset_platform = DataPlatform.SYNAPSE
+
     @staticmethod
     def from_config_file(config_file: str) -> "SynapseExtractor":
         return SynapseExtractor(SynapseConfig.from_yaml_file(config_file))
 
     def __init__(self, config: SynapseConfig):
-        super().__init__(
-            config, "Synapse metadata crawler", Platform.SYNAPSE, DataPlatform.SYNAPSE
-        )
+        super().__init__(config)
         self._config = config
         self._filter = config.filter.normalize()
         self._lookback_days = config.query_log.lookback_days if config.query_log else 0
+        sql_query_endpoint = f"{self._config.server_name}.sql.azuresynapse.net"
+        sql_on_demand_query_endpoint = (
+            f"{self._config.server_name}-ondemand.sql.azuresynapse.net"
+        )
+        self._serverless_client = MssqlClient(
+            sql_on_demand_query_endpoint, self._config.username, self._config.password
+        )
+        self._dedicated_client = MssqlClient(
+            sql_query_endpoint, self._config.username, self._config.password
+        )
+        self._dedicated_databases: Set[str] = set()
 
     async def extract(self) -> Collection[ENTITY_TYPES]:
         logger.info(
             f"Fetching metadata from Synapse workspace: {self._config.server_name}"
         )
 
-        sql_query_endpoint = f"{self._config.server_name}.sql.azuresynapse.net"
-        sql_on_demand_query_endpoint = (
-            f"{self._config.server_name}-ondemand.sql.azuresynapse.net"
-        )
-        serverless_client = MssqlClient(
-            sql_on_demand_query_endpoint, self._config.username, self._config.password
-        )
-        dedicated_client = MssqlClient(
-            sql_query_endpoint, self._config.username, self._config.password
-        )
-        start_date = start_of_day(self._lookback_days)
-        end_date = start_of_day()
         entities: List[ENTITY_TYPES] = []
-        querylog_list: List[QueryLog] = []
 
         # Serverless sqlpool
         try:
-            for database in serverless_client.get_databases():
+            for database in self._serverless_client.get_databases():
                 if not self._filter.include_database(database.name):
                     continue
-                tables = serverless_client.get_tables(database.name)
+                tables = self._serverless_client.get_tables(database.name)
                 datasets = self._map_tables_to_dataset(
                     self._config.server_name, database, tables
                 )
                 self._set_foreign_keys_to_dataset(
-                    datasets, database.name, serverless_client
+                    datasets, database.name, self._serverless_client
                 )
                 entities.extend(datasets.values())
-            if self._lookback_days > 0:
-                querlogs = self._map_query_log(
-                    WorkspaceQuery.get_sql_pool_query_logs(
-                        serverless_client.config,
-                        start_date,
-                        end_date,
-                    )
-                )
-                querylog_list.extend(querlogs)
         except Exception as error:
             logger.exception(f"serverless sqlpool error: {error}")
 
         # Dedicated sqlpool
         try:
-            for database in dedicated_client.get_databases():
+            for database in self._dedicated_client.get_databases():
                 if not self._filter.include_database(database.name):
                     continue
-                tables = dedicated_client.get_tables(database.name)
+                self._dedicated_databases.add(database.name)
+                tables = self._dedicated_client.get_tables(database.name)
                 datasets = self._map_tables_to_dataset(
                     self._config.server_name, database, tables
                 )
                 self._set_foreign_keys_to_dataset(
-                    datasets, database.name, dedicated_client
+                    datasets, database.name, self._dedicated_client
                 )
                 entities.extend(datasets.values())
-                if self._lookback_days > 0:
-                    querlogs = self._map_query_log(
-                        WorkspaceQuery.get_dedicated_sql_pool_query_logs(
-                            dedicated_client.config,
-                            database.name,
-                            start_date,
-                            end_date,
-                        ),
-                        database.name,
-                    )
-                    querylog_list.extend(querlogs)
         except Exception as error:
             logger.exception(f"dedicated sqlpool error: {error}")
 
-        entities.extend(chunk_query_logs(querylog_list))
         return entities
 
+    def collect_query_logs(self) -> Iterator[QueryLog]:
+        """
+        Prerequisite: `extract` must be called before this method is called.
+        """
+        if self._lookback_days <= 0:
+            return
+
+        start_date = start_of_day(self._lookback_days)
+        end_date = start_of_day()
+        mapped_log_count = 0
+
+        for log in self._map_query_log(
+            WorkspaceQuery.get_sql_pool_query_logs(
+                self._serverless_client.config,
+                start_date,
+                end_date,
+            )
+        ):
+            yield log
+            mapped_log_count += 1
+
+        for database in self._dedicated_databases:
+            for log in self._map_query_log(
+                WorkspaceQuery.get_dedicated_sql_pool_query_logs(
+                    self._dedicated_client.config,
+                    database,
+                    start_date,
+                    end_date,
+                ),
+                database,
+            ):
+                yield log
+                mapped_log_count += 1
+
+        logger.info(f"Wrote {mapped_log_count} QueryLog")
+
     def _map_query_log(
         self, rows: Iterable[SynapseQueryLog], database: Optional[str] = None
     ):
@@ -150,16 +165,16 @@ class SynapseExtractor(MssqlExtractor):
         return querylog_map.values()
 
     _query_type_map = {
-        "CREATE": TypeEnum.CREATE_TABLE,
-        "SELECT": TypeEnum.SELECT,
-        "UPDATE": TypeEnum.UPDATE,
-        "DROP": TypeEnum.DROP_TABLE,
-        "ALTER": TypeEnum.ALTER_TABLE,
-        "TRUNCATE": TypeEnum.TRUNCATE,
-        "INSERT": TypeEnum.INSERT,
-        "DELETE": TypeEnum.DELETE,
+        "CREATE": LogType.CREATE_TABLE,
+        "SELECT": LogType.SELECT,
+        "UPDATE": LogType.UPDATE,
+        "DROP": LogType.DROP_TABLE,
+        "ALTER": LogType.ALTER_TABLE,
+        "TRUNCATE": LogType.TRUNCATE,
+        "INSERT": LogType.INSERT,
+        "DELETE": LogType.DELETE,
     }
 
     @staticmethod
-    def _map_query_type(operation: str) -> TypeEnum:
-        return SynapseExtractor._query_type_map.get(operation.upper(), TypeEnum.OTHER)
+    def _map_query_type(operation: str) -> LogType:
+        return SynapseExtractor._query_type_map.get(operation.upper(), LogType.OTHER)
