@@ -7,7 +7,7 @@ from pydantic import BaseModel
 class SynapseQueryLog(BaseModel):
     # transaction_id for serverless sql pool
     request_id: str
-    session_id: Optional[str]
+    session_id: Optional[str] = None
     sql_query: str
     login_name: str
     start_time: datetime
@@ -15,7 +15,7 @@ class SynapseQueryLog(BaseModel):
     # in milliseconds
     duration: int
     # in MB
-    query_size: Optional[int]
-    error: Optional[str]
-    row_count: Optional[int]
-    query_operation: Optional[str]
+    query_size: Optional[int] = None
+    error: Optional[str] = None
+    row_count: Optional[int] = None
+    query_operation: Optional[str] = None
