@@ -20,6 +20,7 @@ query($first: Int, $offset: Int) {
       luid
       name
       projectName
+      projectVizportalUrlId
       vizportalUrlId
       upstreamDatasources {
         id
@@ -41,6 +42,9 @@ query($first: Int, $offset: Int) {
             connectionType
           }
         }
+        owner {
+          luid
+        }
       }
       embeddedDatasources {
         id
@@ -60,6 +64,9 @@ query($first: Int, $offset: Int) {
           }
         }
       }
+      tags {
+        name
+      }
     }
   }
 }
@@ -68,22 +75,26 @@ query($first: Int, $offset: Int) {
 
 class Database(BaseModel):
     name: str
-    description: Optional[str]
+    description: Optional[str] = None
     connectionType: str
 
 
 class DatabaseTable(BaseModel):
     luid: str
     name: str
-    fullName: Optional[str]
-    schema_: Optional[str] = Field(alias="schema")
-    description: Optional[str]
-    database: Optional[Database]
+    fullName: Optional[str] = None
+    schema_: Optional[str] = Field(None, alias="schema")
+    description: Optional[str] = None
+    database: Optional[Database] = None
 
 
 class DatasourceField(BaseModel):
     name: str
-    description: Optional[str]
+    description: Optional[str] = None
+
+
+class Owner(BaseModel):
+    luid: str
 
 
 class PublishedDatasource(BaseModel):
@@ -91,9 +102,10 @@ class PublishedDatasource(BaseModel):
     luid: str
     vizportalUrlId: str
     name: str
-    description: Optional[str]
+    description: Optional[str] = None
     fields: List[DatasourceField]
     upstreamTables: List[DatabaseTable]
+    owner: Owner
 
 
 class EmbeddedDatasource(BaseModel):
@@ -103,15 +115,21 @@ class EmbeddedDatasource(BaseModel):
     upstreamTables: List[DatabaseTable]
 
 
+class Tag(BaseModel):
+    name: str
+
+
 class WorkbookQueryResponse(BaseModel):
     """Modeling Metadata Graphql API response for a workbook"""
 
     luid: str
     name: str
     projectName: str
+    projectVizportalUrlId: str
     vizportalUrlId: str
     upstreamDatasources: List[PublishedDatasource]
     embeddedDatasources: List[EmbeddedDatasource]
+    tags: List[Tag]
 
 
 # GraphQL that lists all custom SQL queries used in datasources.
