@@ -1,6 +1,7 @@
 """
 This module models the ThoughtSpot SDK complex return objects.
 """
+
 from enum import Enum
 from typing import Dict, List, Optional
 
@@ -17,9 +18,12 @@ class Tag(BaseModel):
 class Header(BaseModel):
     id: str
     name: str
-    description: Optional[str]
+    description: Optional[str] = None
     tags: List[Tag] = []
 
+    def __repr__(self):
+        return self.id
+
 
 class Reference(BaseModel):
     id: str
@@ -34,9 +38,9 @@ class Metadata(BaseModel):
 
 
 class DataSourceConfiguration(BaseModel):
-    accountName: Optional[str]
-    user: Optional[str]
-    project_id: Optional[str]
+    accountName: Optional[str] = None
+    user: Optional[str] = None
+    project_id: Optional[str] = None
 
 
 class DataSourceContent(BaseModel):
@@ -51,10 +55,11 @@ class TableMappingInfo(BaseModel):
 
 
 class LogicalTableContent(BaseModel):
-    physicalTableName: Optional[str]
+    physicalTableName: Optional[str] = None
     worksheetType: str
     joinType: str
-    tableMappingInfo: Optional[TableMappingInfo]
+    tableMappingInfo: Optional[TableMappingInfo] = None
+    sqlQuery: Optional[str] = None
 
 
 class ConnectionType(Enum):
@@ -81,21 +86,22 @@ class ColumnMappingInfo(BaseModel):
 
 class ColumnMetadata(Metadata):
     type: str
-    columnMappingInfo: Optional[ColumnMappingInfo]
+    columnMappingInfo: Optional[ColumnMappingInfo] = None
     sources: List[ColumnSource]
-    dataType: Optional[str]
-    optionalType: Optional[str]
+    dataType: Optional[str] = None
+    optionalType: Optional[str] = None
 
 
 class SourceType(Enum):
     WORKSHEET = "WORKSHEET"
     ONE_TO_ONE_LOGICAL = "ONE_TO_ONE_LOGICAL"
     AGGR_WORKSHEET = "AGGR_WORKSHEET"
+    SQL_VIEW = "SQL_VIEW"
 
 
 class VizColumn(BaseModel):
-    referencedTableHeaders: Optional[List[Reference]]
-    referencedColumnHeaders: Optional[List[Reference]]
+    referencedTableHeaders: Optional[List[Reference]] = None
+    referencedColumnHeaders: Optional[List[Reference]] = None
 
 
 class RefAnswerBook(BaseModel):
@@ -104,9 +110,9 @@ class RefAnswerBook(BaseModel):
 
 class VizContent(BaseModel):
     vizType: str
-    chartType: Optional[str]
-    refVizId: Optional[str]
-    refAnswerBook: Optional[RefAnswerBook]
+    chartType: Optional[str] = None
+    refVizId: Optional[str] = None
+    refAnswerBook: Optional[RefAnswerBook] = None
     columns: List[VizColumn] = []
 
 
@@ -124,8 +130,8 @@ class Question(BaseModel):
 
 class Sheet(Metadata):
     sheetContent: SheetContent
-    sheetType: Optional[str]
-    question: Optional[Question]
+    sheetType: Optional[str] = None
+    question: Optional[Question] = None
 
 
 class ReportContent(BaseModel):
@@ -140,62 +146,71 @@ class LiveBoardHeader(Header):
     resolvedObjects: Dict[str, ResolvedObject]
 
 
-class ConnectionHeader(Header):
-    type: str
-    authorName: str = ""
-
-
 class LogicalTable(Metadata):
     columns: List[ColumnMetadata]
     logicalTableContent: LogicalTableContent
 
 
-class ConnectionMetadata(Metadata):
-    header: Header
+class ConnectionDetail(Header):
     type: ConnectionType
-    dataSourceContent: DataSourceContent
-    logicalTableList: List[LogicalTable]
+    configuration: str
 
 
-class SourceMetadata(Metadata):
-    type: SourceType
+class Connection(BaseModel):
+    details: ConnectionDetail
+
+
+class LogicalTableMetadataDetail(Metadata):
+    type: str
     columns: List[ColumnMetadata]
     dataSourceId: str
     dataSourceTypeEnum: DataSourceTypeEnum
     logicalTableContent: LogicalTableContent
 
 
-class AnswerMetadata(Metadata):
+class LogicalTableMetadata(BaseModel):
+    metadata_detail: LogicalTableMetadataDetail
+
+
+class AnswerMetadataDetail(Metadata):
     type: str
     reportContent: ReportContent
 
 
-class LiveBoardMetadata(Metadata):
+class AnswerMetadata(BaseModel):
+    metadata_detail: AnswerMetadataDetail
+
+
+class LiveBoardMetadataDetail(Metadata):
     header: LiveBoardHeader
     type: str
     reportContent: ReportContent
 
 
+class LiveBoardMetadata(BaseModel):
+    metadata_detail: LiveBoardMetadataDetail
+
+
 class TMLResult(BaseModel):
     info: Header
-    edoc: Optional[str]
+    edoc: Optional[str] = None
 
 
 class TMLFormula(BaseModel):
-    id: Optional[str]
+    id: Optional[str] = None
     name: str
     expr: str
 
 
 class TMLBase(BaseModel):
     name: str
-    formulas: List[TMLFormula]
+    formulas: Optional[List[TMLFormula]] = None
 
 
 class TMLColumn(BaseModel):
     name: str
-    column_id: Optional[str]
-    formula_id: Optional[str]
+    column_id: Optional[str] = None
+    formula_id: Optional[str] = None
 
 
 class TMLWorksheet(TMLBase):
@@ -206,7 +221,23 @@ class TMLView(TMLBase):
     view_columns: List[TMLColumn]
 
 
+class TMLTable(BaseModel):
+    name: str
+    id: Optional[str] = None
+    fqn: Optional[str] = None
+
+
+class TMLAnswerTableObject(BaseModel):
+    ordered_column_ids: List[str]
+
+
+class TMLAnswer(TMLBase):
+    tables: List[TMLTable]
+    table: TMLAnswerTableObject
+
+
 class TMLObject(BaseModel):
     guid: str
-    worksheet: Optional[TMLWorksheet]
-    view: Optional[TMLView]
+    worksheet: Optional[TMLWorksheet] = None
+    view: Optional[TMLView] = None
+    answer: Optional[TMLAnswer] = None
